/*
Cory McKay

PeakList Class

- Objects of this class are nodes in a list that is used to store the indices of peaks that have
been located.
- The first object of this class should be created with the PeakList() constructor.
- Peaks can then be added to the list with the insert_new_peak(int) method.
- The number of peaks added so far can be accessed with the get_number_of_peaks() method.
- The peak indices found so far can be accessed with the get_peak_indices() method.
*/

class PeakList 
{
	// Fields
	private int current_peak; // the index of the peak recorded at the current node
	private PeakList previous_node; // the previous node in the list. Is null when there are no nodes
	                                // before the current node. 
	private PeakList next_node; // the next node in the list. Is null when there are no nodes
	                            // after the current node. 
	private static PeakList front_of_queue; // the item that was first inserted into the queue
	private static PeakList end_of_queue; // the item that has most recently been inserted into the queue
	private static int size_of_queue; // the total number of peak indices that have been stored in the queue


	/***************************************************************************/


	PeakList()
	// Constructor
	// Creates an empty node with current_peak set to -1.
	// Should be used to create the first node.
	{
		current_peak = -1;
		previous_node = null;
		next_node = null;
		front_of_queue = this;
		end_of_queue = this;
		size_of_queue = 0;
	}


	/***************************************************************************/


	PeakList(int peak_index, PeakList old_node)
	// Constructor
	// Creates a new node that contains a peak index peak_index.
	// Should be used to create all but the first node.
	// Should only be used by objects of the PeakList class.
	{
		current_peak = peak_index;
		previous_node = null;
		next_node = old_node;
		old_node.previous_node = this;
		size_of_queue++;
	}


	/***************************************************************************/


	public static void insert_new_peak(int peak_index)
	// Inserts a node at the end of the list with the specified peak_index.
	// If there is only one node and it is blank, then gives it the specified contents instead.
	{
		if (front_of_queue.current_peak == -1)
		{
			front_of_queue.current_peak = peak_index;
			size_of_queue++;
		}
		else if (front_of_queue.previous_node != null)
			end_of_queue = new PeakList(peak_index, end_of_queue);
		else
			end_of_queue = new PeakList(peak_index, front_of_queue);
	}


	/***************************************************************************/


	public int get_number_of_peaks()
	// Returns the total number of peak indices that have been added to the list.
	{
		return size_of_queue;
	}


	/***************************************************************************/


	public int[] get_peak_indices()
	// Returns an integer array containing all peak indices that have been added to the list.
	{
		int[] indices = new int[size_of_queue];
		PeakList current_node = front_of_queue;
		for (int i = 0; i < size_of_queue; i++)
		{
			indices[i] = current_node.current_peak;
			current_node = current_node.previous_node;
		}
		return indices;
	}
}