/*
Cory McKay

PeakinessList Class

- Objects of this class are nodes in a list that is used to store indices of valleys and
  the peakiness values that correspond to them.
- The first object of this class should be created with the PeakinessList constructor.
- Nodes can then be added to the list with the insert_new_node method.
- The number of nodes added so far can be accessed with the get_number_of_nodes method.
- The PeakinessObjects found so far can be accessed with the get_peakiness_objects method.
*/

class PeakinessList 
{
	// Fields
	private PeakinessObject node_contents; // the data for the node. Is null when empty.
	private PeakinessList previous_node; // the previous node in the list. Is null when there are no nodes
	                                // before the current node. 
	private PeakinessList next_node; // the next node in the list. Is null when there are no nodes
	                            // after the current node. 
	private static PeakinessList front_of_queue; // the item that was first inserted into the queue
	private static PeakinessList end_of_queue; // the item that has most recently been inserted into the queue
	private static int size_of_queue; // the total number of nodes that have been stored in the queue


	/***************************************************************************/


	PeakinessList()
	// Constructor
	// Creates an empty node with node_contents set to null.
	// Should be used to create the first node.
	{
		node_contents = null;
		previous_node = null;
		next_node = null;
		front_of_queue = this;
		end_of_queue = this;
		size_of_queue = 0;
	}


	/***************************************************************************/


	PeakinessList(int valley_indice, double peakiness, PeakinessList old_node)
	// Constructor
	// Creates a new node that contains the indice of a valley and the corresponding peakiness.
	// Should be used to create all but the first node.
	// Should only be used by objects of the PeakinessList class.
	{
		node_contents = new PeakinessObject(valley_indice, peakiness);
		previous_node = null;
		next_node = old_node;
		old_node.previous_node = this;
		size_of_queue++;
	}


	/***************************************************************************/


	public static void insert_new_node(int valley_indice, double peakiness)
	// Inserts a node at the end of the list with the specified valley_index and the corresponding
	// peakiness.
	// If there is only one node and it is blank, then gives it the specified contents instead.
	{
		if (front_of_queue.node_contents == null)
		{
			front_of_queue.node_contents = new PeakinessObject(valley_indice, peakiness);
			size_of_queue++;
		}
		else if (front_of_queue.previous_node != null)
			end_of_queue = new PeakinessList(valley_indice, peakiness, end_of_queue);
		else
			end_of_queue = new PeakinessList(valley_indice, peakiness, front_of_queue);
	}


	/***************************************************************************/


	public int get_number_of_nodes()
	// Returns the total number of nodes that have been added to the list.
	{
		return size_of_queue;
	}


	/***************************************************************************/


	public PeakinessObject[] get_peakiness_objects()
	// Returns PeakinessObject array containing all nodes that have been added to the list.
	{
		PeakinessObject[] nodes = new PeakinessObject[size_of_queue];
		PeakinessList current_node = front_of_queue;
		for (int i = 0; i < size_of_queue; i++)
		{
			nodes[i] = current_node.node_contents;
			current_node = current_node.previous_node;
		}
		return nodes;
	}
}